{Aufbau des DMX-Dongles:
 Gegenber der Schaltung aus dem Internet wurden folgende
 funktionalen und Schaltungsvernderungen vorgenommen:
 PORTA 0 / Pin 17 / Ausgang - (wie bisher)
 PORTA 1 / Pin 18 / Eingang - AUTOFEED / Pin 14 / mit Pull-Up
 PORTA 2 / Pin 1  / Ausgang - BUSY&ACK / Pin 10&11
 PORTA 3 / Pin 2  / Eingang - STROBE / Pin 1 / mit Pull-Up
 PORTA 4 / Pin 3  / Eingang - SELIN / Pin 17 / mit Pull-Down
 PORTB / Pin 6..13 / Ein+Aus - DATA / Pin 2..9

 Das neue Mikrocontroller-Programm hat folgende Features:
 * Steuerbarkeit von 64 Kanlen (wie bisher)
 * Speicherung der Kanal-Bytes im EEPROM
 * automatische Restauration der Kanal-Bytes beim Einschalten
 * DMX-Mindestwiederholrate 50Hz, egal bei welchem LPT-Zustand
   (via Watchdog-Timer realisiert)
 * przises Ein-Flanken-Handshaking zur Datenbergabe
 * Rcklese-Mglichkeit ber bidirektionales Parallelport
   (z.Z. ungetestet)

 Die Richtungsumschaltung (lesen oder einschreiben) erfolgt
 mit der SELIN-Leitung, LOW=schreiben, HIGH=lesen.
 Mit AUTOFEED=HIGH wird der Mikrocontroller in einen Modus
 versetzt, bei dem er "computer-hrig" wird, und quittiert
 die Empfangsbereitschaft mit BUSY=LOW. Whrend dieser
 Zeit kann er kein DMX-Signal aussenden! Bei berschreitung
 der Zeit von ca. 18 ms wird automatisch BUSY=HIGH gesetzt
 und ein DMX-Burst gesendet.

 Mit AUTOFEED=LOW ist das Computer-Interface "taub", BUSY=HIGH,
 der Adressen-Zhler zurckgesetzt.
 Der Mikrocontroller repetiert mit seiner maximalen Frequenz.
 Nebenbei gleicht er Schritt fr Schritt RAM- und EPROM-Inhalt
 ab.

 Solange BUSY=LOW ist, kann mit STROBE=LOW ein Byte gelesen
 bzw. eingeschrieben werden; dabei wird mit BUSY=HIGH quittiert.
 Mit der Rcknahme von STROBE nach HIGH wird auch BUSY=LOW.
 Somit kann die bertragung jedes Bytes sauber erfat werden,
 auch wenn der Computer schneller als der Mikrocontroller sein
 sollte.

 Vor dem Belegen von SELIN=HIGH sollte das LPT auf Eingabe
 geschaltet werden, z.B. mit dem PS/2-Modus (im ECP-Steuerregister
 PortBase+$402 einzustellen) mit dem Bit 5 von PortBase+2.
}
{$k+}
{$r dmx00}
uses winprocs,wintypes,win31;
type
 LongRec=record
  lo,hi:Word;
 end;
const
 PortBasis: Word=$378;                  {LPT1}
 Basisadr: Integer=1;
var
 Kanaele: array[0..6] of Byte;
 CatchBuf: TCatchBuf;
{$S-,R-}

procedure busywait;
 var
  i:Integer;
 begin
  for i:=10000 downto 0 do begin
   if port[PortBasis+1] and $80 <>0 then exit;	{warten bis BUSY=LOW}
  end;
  Throw(CatchBuf,1);
 end;

procedure putbyte(b:Byte);
 begin
  busywait;
  port[PortBasis+0]:=b;
  port[PortBasis+2]:=port[PortBasis+2] and $1F or $01;{STROBE=low}
  while port[PortBasis+1] and $80 <>0 do;	{EXTRA: warten bis BUSY=HIGH}
  port[PortBasis+2]:=port[PortBasis+2] and $1E;	{STROBE=high}
 end;

procedure PutKanaele;
 var
  i:Integer;
 begin
  if Catch(CatchBuf)=0 then begin
   port[PortBasis+2]:=port[PortBasis+2] and $1D or $08;   {AUTOFEED=HIGH, SEL=LOW}
   for i:=1 to Basisadr-1 do PutByte(0);
   for i:=0 to 6 do PutByte(Kanaele[i]);
   for i:=Basisadr+7 to 63 do PutByte(0);
   port[PortBasis+2]:=port[PortBasis+2] and $1F or $02;        {AUTOFEED=LOW}
  end;
 end;

function GetByte:Byte;
 begin
  busywait;
  port[PortBasis+2]:=port[PortBasis+2] and $1F or $01;{STROBE=low}
  GetByte:=port[PortBasis+0];
  while port[PortBasis+1] and $80 <>0 do;	{EXTRA: warten bis BUSY=HIGH}
  port[PortBasis+2]:=port[PortBasis+2] and $1E;	{STROBE=high}
 end;

function GetKanaele:Boolean;
 var
  i: Integer;
 begin
  GetKanaele:=false;
  if Catch(CatchBuf)=0 then begin
   port[PortBasis+$402]:=20;		{PS/2-Modus}
   port[PortBasis+2]:=port[PortBasis+2] and $15 or $20;   {AUTOFEED=HIGH, SEL=HIGH, INPUT}
   for i:=1 to Basisadr-1 do GetByte;
   for i:=0 to 6 do Kanaele[i]:=GetByte;
{  for i:=Basisadr+7 to 63 do GetByte;}
   port[PortBasis+2]:=port[PortBasis+2] and $1F or $0A; {AUTOFEED=LOW, SEL=LOW, OUTPUT}
   GetKanaele:=true;
  end;
 end;

procedure ShowPos(Wnd:HWnd; NewPos:Integer);
{Setzt die Position im Edit-Fenster und am DMX-Dongle, mithin am Scanner
 Wnd ist das Handle des sendenden Rollbalkens}
 var
  ID: Word;
 begin
  ID:=GetDlgCtrlID(Wnd);
  SetDlgItemInt(GetParent(Wnd),ID+7,NewPos,true);
  Kanaele[ID-101]:=NewPos;
  PutKanaele;
 end;

procedure Roll(Wnd:HWnd; RollDiff:Integer);
 begin
  Inc(RollDiff,GetScrollPos(Wnd,SB_Ctl));
  if RollDiff<0 then RollDiff:=0;
  if RollDiff>255 then RollDiff:=255;
  SetScrollPos(Wnd,SB_Ctl,RollDiff,true);
  ShowPos(Wnd,RollDiff);
 end;

function MainDlgProc(wnd:hwnd;msg,wparam:word;lparam:longint):bool;export;
 var
  lPar: LongRec absolute lParam;
  i: Integer;
  w: HWnd;
 begin
  MainDlgProc:=false;
  case msg of
   WM_InitDialog: begin
    if not GetKanaele then MessageBox(wnd,'Box reagiert nicht',nil,0);
    for i:=0 to 6 do begin
     w:=GetDlgItem(Wnd,i+101);
     SetScrollRange(w,SB_Ctl,0,255,false);
     SetScrollPos(w,SB_Ctl,Kanaele[i],false);
     SetDlgItemInt(Wnd,i+108,Kanaele[i],true);
    end;
    MainDlgProc:=true;
   end;
   WM_Command: case wParam of
    2: EndDialog(wnd,0);
   end;
   WM_VScroll: case wparam of
    SB_LineUp: Roll(lPar.hi,-4);
    SB_LineDown: Roll(lPar.hi,4);
    SB_PageUp: Roll(lPar.hi,-32);
    SB_PageDown: Roll(lPar.hi,32);
    SB_ThumbTrack,SB_ThumbPosition: begin
     SetScrollPos(lPar.hi,SB_Ctl,lPar.lo,true);
     ShowPos(lPar.hi,lPar.lo);
    end;
   end;
  end;
 end;

begin
{ if VCall0_Entry=nil
 then MessageBox(0,'Kein Ring0 heute.','erstes Programm',0);}
 DialogBox(HInstance, MakeIntResource(100), 0, @MainDlgProc);
end.